@@ -48,153 +48,6 @@ struct pevt_stage1
 };
 
 
-/* Make ip:port urls clikable (127.0.0.1:80 etc)
-*  patch by Alex <alex@cosinus.org> & dobler <dobler@barrysworld.com>
-*/
-
-static int
-q3link (char *word)
-{
-	char *s;
-	int i;
-	int d = 0;
-
-	if ((s = strchr (word,':')) != NULL)
-	{
-		for (i = 0; i < s - word; i++)
-		{
-			if (word[i] == '.')
-				d++;
-			else if (!isdigit (word[i]))
-			{
-				d = 0;
-				break;
-			}
-		}
-
-		if (d == 3)
-		{
-			s++;
-			d = 0;
-
-			while (*s != 0)
-			{
-				if (!isdigit (*s++))
-					return (0);
-				d++;
-			}
-			if (d > 0)
-				return (1);
-		}
-	}
-	return (0);
-}
-
-/* check if a word is clickable */
-
-int
-text_word_check (char *word)
-{
-	session *sess = current_sess;
-	char *at, *dot;
-	int i, dots;
-	int len = strlen (word);
-
-	if(q3link(word))
-		return WORD_URL;
-
-	if ((word[0] == '@' || word[0] == '+' || word[0] == '^' || word[0] == '%' || word[0] == '*' ) && word[1] == '#')
-		return WORD_CHANNEL;
-
-	if ((word[0] == '#' || word[0] == '&') && word[1] != '#' && word[1] != 0)
-		return WORD_CHANNEL;
-
-	if (!strncasecmp (word, "irc.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "ftp.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "www.", 4) && word[4] != '.')
-		return WORD_URL;
-
-	if (!strncasecmp (word, "irc://", 6) && word[6] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "ftp://", 6) && word[6] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "http://", 7) && word[7] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "file://", 7) && word[7] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "https://", 8) && word[8] != 0)
-		return WORD_URL;
-
-	if (!strncasecmp (word, "gopher://", 9) && word[9] != 0)
-		return WORD_URL;
-
-	if (( (word[0]=='@' || word[0]=='+') && find_name (sess, word+1)) || find_name (sess, word))
-		return WORD_NICK;
-
-	at = strchr (word, '@');	  /* check for email addy */
-	dot = strrchr (word, '.');
-	if (at && dot)
-	{
-		if (at < dot)
-		{
-			if (strchr (word, '*'))
-				return WORD_HOST;
-			else
-				return WORD_EMAIL;
-		}
-	}
- 
-	/* check if it's an IP number */
-	dots = 0;
-	for (i = 0; i < len; i++)
-	{
-		if (word[i] == '.' && i > 1)
-			dots++;
-		else if (!isdigit (word[i]))
-		{
-			dots = 0;
-			break;
-		}
-	}
-	if (dots == 3)
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 5, ".html", 5))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".org", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".net", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".com", 4))
-		return WORD_HOST;
-
-	if (!strncasecmp (word + len - 4, ".edu", 4))
-		return WORD_HOST;
-
-	if (len > 5)
-	{
-		if (word[len - 3] == '.' &&
-			 isalpha (word[len - 2]) && isalpha (word[len - 1]))
-			return WORD_HOST;
-	}
-
-	if (sess->type == SESS_DIALOG)
-		return WORD_DIALOG;
-
-	return 0;
-}
-
 void
 log_close (session *sess)
 {
@@ -213,117 +66,201 @@ log_close (session *sess)
 }
 
 static void
-log_create_filename (char *buf, char *servname, char *channame, char *netname)
+mkdir_p (char *dir)	/* like "mkdir -p" from a shell, FS encoding */
 {
-	char fn[256];
-	char *tmp, *dir, *sep;
-	int pathlen=510, c=0, mbl;
+	char *start = dir;
 
-	if (!rfc_casecmp (channame, servname))
+	/* the whole thing already exists? */
+	if (access (dir, F_OK) == 0)
+		return;
+
+	while (*dir)
 	{
-		channame = g_strdup ("server");
-	} else
-	{
-		sep = tmp = strdup (channame);
-		while (*tmp)
-		{
-			mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
-			if (mbl == 1)
-			{
-#ifndef WIN32
-				*tmp = rfc_tolower (*tmp);
-				if (*tmp == '/')
-#else
-				/* win32 can't handle filenames with \|/>< characters */
-				if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
-					 *tmp == '>' || *tmp == '<')
-#endif
-					*tmp = '_';
-			}
-			tmp += mbl;
-		}
-
-		channame = g_filename_from_utf8 (sep, -1, 0, 0, 0);
-		free (sep);
-	}
-
-	snprintf (buf, 512, "%s/xchatlogs", get_xdir_fs ());
-	if (access (buf, F_OK) != 0)
 #ifdef WIN32
-		mkdir (buf);
+		if (dir != start && (*dir == '/' || *dir == '\\'))
 #else
-		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
+		if (dir != start && *dir == '/')
 #endif
-	auto_insert (fn, sizeof (fn), prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
-	g_free (channame);
-
-	snprintf (buf, 512, "%s/xchatlogs/%s", get_xdir_fs (), fn);
-
- 	/* The following code handles subdirectories in logpath and creates
- 	 * them, if they don't exist. Useful with logmasks like "%c/%y.log" in
- 	 * ~/.xchat/xchat.conf.
- 	 *                       -- patch by tvk <tvk@von-koch.com>
- 	 */
-
-	if (access (buf, F_OK) != 0)
-	{
-		snprintf (buf, 512, "%s/xchatlogs/", get_xdir_fs ());
-		pathlen -= strlen(buf);
- 
-		/* how many sub-directories do we have? */
-		sep = fn;
-		while (*sep++) if (*sep=='/') c++;
-			
-		if (c) {
- 			/* create each sub-directory */
-			sep = strdup(fn);
-			dir = strtok(sep, "/");
-		
-			while (c-- && dir != NULL) {
-				strncat (buf, dir, pathlen);
-				strcat (buf, "/");
-				pathlen -= strlen(dir);
- 		
-				if (access (buf, F_OK) != 0)
-#ifdef WIN32	
-					mkdir (buf);
-#else	
-					mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
-#endif	
-				dir = strtok(NULL, "/");
-			}
-
-			/* append the filename */
-			strncat (buf, strrchr(fn, '/')+1, pathlen);
-			free (sep);
-		} else {
-			strncat (buf, fn, pathlen);
+		{
+			*dir = 0;
+#ifdef WIN32
+			mkdir (start);
+#else
+			mkdir (start, S_IRUSR | S_IWUSR | S_IXUSR);
+#endif
+			*dir = '/';
 		}
-
+		dir++;
 	}
 }
 
+static char *
+log_create_filename (char *channame)
+{
+	char *tmp, *ret;
+	int mbl;
+
+	ret = tmp = strdup (channame);
+	while (*tmp)
+	{
+		mbl = g_utf8_skip[((unsigned char *)tmp)[0]];
+		if (mbl == 1)
+		{
+#ifndef WIN32
+			*tmp = rfc_tolower (*tmp);
+			if (*tmp == '/')
+#else
+			/* win32 can't handle filenames with \|/><:"*? characters */
+			if (*tmp == '\\' || *tmp == '|' || *tmp == '/' ||
+				 *tmp == '>'  || *tmp == '<' || *tmp == ':' ||
+				 *tmp == '\"' || *tmp == '*' || *tmp == '?')
+#endif
+				*tmp = '_';
+		}
+		tmp += mbl;
+	}
+
+	return ret;
+}
+
+/* like strcpy, but % turns into %% */
+
+static char *
+log_escape_strcpy (char *dest, char *src, char *end)
+{
+	while (*src)
+	{
+		*dest = *src;
+		if (dest + 1 == end)
+			break;
+		dest++;
+		src++;
+
+		if (*src == '%')
+		{
+			if (dest + 1 == end)
+				break;
+			dest[0] = '%';
+			dest++;
+		}
+	}
+
+	dest[0] = 0;
+	return dest - 1;
+}
+
+/* substitutes %c %n %s into buffer */
+
+static void
+log_insert_vars (char *buf, int bufsize, char *fmt, char *c, char *n, char *s)
+{
+	char *end = buf + bufsize;
+
+	while (1)
+	{
+		switch (fmt[0])
+		{
+		case 0:
+			buf[0] = 0;
+			return;
+
+		case '%':
+			fmt++;
+			switch (fmt[0])
+			{
+			case 'c':
+				buf = log_escape_strcpy (buf, c, end);
+				break;
+			case 'n':
+				buf = log_escape_strcpy (buf, n, end);
+				break;
+			case 's':
+				buf = log_escape_strcpy (buf, s, end);
+				break;
+			default:
+				buf[0] = '%';
+				buf++;
+				buf[0] = fmt[0];
+				break;
+			}
+			break;
+
+		default:
+			buf[0] = fmt[0];
+		}
+		fmt++;
+		buf++;
+		/* doesn't fit? */
+		if (buf == end)
+		{
+			buf[-1] = 0;
+			return;
+		}
+	}
+}
+
+static char *
+log_create_pathname (char *servname, char *channame, char *netname)
+{
+	char fname[384];
+	char fnametime[384];
+	char *fs;
+	struct tm *tm;
+	time_t now;
+
+	if (!netname)
+		netname = "NETWORK";
+
+	/* first, everything is in UTF-8 */
+	if (!rfc_casecmp (channame, servname))
+		channame = strdup ("server");
+	else
+		channame = log_create_filename (channame);
+	log_insert_vars (fname, sizeof (fname), prefs.logmask, channame, netname, servname);
+	free (channame);
+
+	/* insert time/date */
+	now = time (NULL);
+	tm = localtime (&now);
+	strftime (fnametime, sizeof (fnametime), fname, tm);
+
+	/* create final path/filename */
+	snprintf (fname, sizeof (fname), "%s/xchatlogs/%s", get_xdir_utf8 (), fnametime);
+
+	/* now we need it in FileSystem encoding */
+	fs = g_filename_from_utf8 (fname, -1, 0, 0, 0);
+
+	/* create all the subdirectories */
+	if (fs)
+		mkdir_p (fs);
+
+	return fs;
+}
+
 static int
 log_open_file (char *servname, char *channame, char *netname)
 {
 	char buf[512];
 	int fd;
+	char *file;
 	time_t currenttime;
 
-	if (!netname)
-		netname = "NETWORK";
+	file = log_create_pathname (servname, channame, netname);
+	if (!file)
+		return -1;
 
-	log_create_filename (buf, servname, channame, netname);
 #ifdef WIN32
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, S_IREAD|S_IWRITE);
 #else
-	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY, 0644);
+	fd = open (file, O_CREAT | O_APPEND | O_WRONLY, 0644);
 #endif
+	g_free (file);
+
 	if (fd == -1)
 		return -1;
 	currenttime = time (NULL);
 	write (fd, buf,
-			 snprintf (buf, 510, _("**** BEGIN LOGGING AT %s\n"),
+			 snprintf (buf, sizeof (buf), _("**** BEGIN LOGGING AT %s\n"),
 						  ctime (&currenttime)));
 
 	return fd;
@@ -344,7 +281,7 @@ log_open (session *sess)
 		snprintf (message, sizeof (message),
 					_("* Can't open log file(s) for writing. Check the\n" \
 					  "  permissions on %s/xchatlogs"), get_xdir_utf8 ());
-		fe_message (message, TRUE);
+		fe_message (message, FE_MSG_WAIT | FE_MSG_ERROR);
 
 		log_error = TRUE;
 	}
@@ -388,10 +325,25 @@ log_write (session *sess, char *text)
 {
 	char *temp;
 	char *stamp;
+	char *file;
 	int len;
 
 	if (sess->logfd != -1 && prefs.logging)
 	{
+		/* change to a different log file? */
+		file = log_create_pathname (sess->server->servername, sess->channel,
+											 server_get_network (sess->server, FALSE));
+		if (file)
+		{
+			if (access (file, F_OK) != 0)
+			{
+				close (sess->logfd);
+				sess->logfd = log_open_file (sess->server->servername, sess->channel,
+													  server_get_network (sess->server, FALSE));
+			}
+			g_free (file);
+		}
+
 		if (prefs.timestamp_logs)
 		{
 			len = get_stamp_str (prefs.timestamp_log_format, time (0), &stamp);
@@ -402,7 +354,11 @@ log_write (session *sess, char *text)
 			}
 		}
 		temp = strip_color (text, -1, 1, 1);
-		write (sess->logfd, temp, strlen (temp));
+		len = strlen (temp);
+		write (sess->logfd, temp, len);
+		/* lots of scripts/plugins print without a \n at the end */
+		if (temp[len - 1] != '\n')
+			write (sess->logfd, "\n", 1);	/* emulate what xtext would display */
 		free (temp);
 	}
 }
@@ -570,6 +526,7 @@ static char * const pevt_join_help[] = {
 static char * const pevt_chanaction_help[] = {
 	N_("Nickname"),
 	N_("The action"),
+	N_("Mode char"),
 };
 
 static char * const pevt_chanmsg_help[] = {
@@ -869,10 +826,11 @@ static char * const pevt_sconnect_help[] = {
 
 static char * const pevt_generic_nick_help[] = {
 	N_("Nickname"),
+	N_("Server Name")
 };
 
 static char * const pevt_chanmodes_help[] = {
-	N_("Channel name"),
+	N_("Channel Name"),
 	N_("Modes string"),
 };
 
@@ -985,7 +943,7 @@ static char * const pevt_notifynumber_help[] = {
 };
 
 static char * const pevt_serverlookup_help[] = {
-	N_("Servername"),
+	N_("Server Name"),
 };
 
 static char * const pevt_servererror_help[] = {
@@ -1075,7 +1033,7 @@ pevent_make_pntevts ()
 		{
 			snprintf (out, sizeof (out),
 						 _("Error parsing event %s.\nLoading default."), te[i].name);
-			fe_message (out, FALSE);
+			fe_message (out, FE_MSG_WARN);
 			free (pntevts_text[i]);
 			len = strlen (te[i].def) + 1;
 			pntevts_text[i] = malloc (len);
@@ -1277,13 +1235,17 @@ load_text_events ()
 	pevent_make_pntevts ();
 }
 
-static void
-display_event (char *i, session *sess, int numargs, char **args)
+void
+format_event (session *sess, int index, char **args, char *o, int sizeofo)
 {
-	int len, oi, ii;
-	char *ar, o[4096], d, a, done_all = FALSE;
+	int len, oi, ii, numargs;
+	char *i, *ar, d, a, done_all = FALSE;
+
+	i = pntevts[index];
+	numargs = te[index].num_args;
 
 	oi = ii = len = d = a = 0;
+	o[0] = 0;
 
 	if (i == NULL)
 		return;
@@ -1296,9 +1258,10 @@ display_event (char *i, session *sess, int numargs, char **args)
 		case 0:
 			memcpy (&len, &(i[ii]), sizeof (int));
 			ii += sizeof (int);
-			if (oi + len > sizeof (o))
+			if (oi + len > sizeofo)
 			{
 				printf ("Overflow in display_event (%s)\n", i);
+				o[0] = 0;
 				return;
 			}
 			memcpy (&(o[oi]), &(i[ii]), len);
@@ -1349,8 +1312,16 @@ display_event (char *i, session *sess, int numargs, char **args)
 	}
 	o[oi] = 0;
 	if (*o == '\n')
-		return;
-	PrintText (sess, o);
+		o[0] = 0;
+}
+
+static void
+display_event (session *sess, int event, char **args)
+{
+	char o[4096];
+	format_event (sess, event, args, o, sizeof (o));
+	if (o[0])
+		PrintText (sess, o);
 }
 
 int
@@ -1404,7 +1375,7 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 		}
 		if (ii == len)
 		{
-			fe_message ("String ends with a $", FALSE);
+			fe_message ("String ends with a $", FE_MSG_WARN);
 			return 1;
 		}
 		d = i[ii++];
@@ -1432,10 +1403,10 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 			continue;
 
 		 a_len_error:
-			fe_message ("String ends in $a", FALSE);
+			fe_message ("String ends in $a", FE_MSG_WARN);
 			return 1;
 		 a_range_error:
-			fe_message ("$a value is greater then 255", FALSE);
+			fe_message ("$a value is greater than 255", FE_MSG_WARN);
 			return 1;
 		}
 		if (d == 't')
@@ -1458,7 +1429,7 @@ pevt_build_string (const char *input, char **output, int *max_arg)
 		if (d < '1' || d > '9')
 		{
 			snprintf (o, sizeof (o), "Error, invalid argument $%c\n", d);
-			fe_message (o, FALSE);
+			fe_message (o, FE_MSG_WARN);
 			return 1;
 		}
 		d -= '0';
@@ -1561,7 +1532,7 @@ text_emit (int index, session *sess, char *a, char *b, char *c, char *d)
 
 	/* If a plugin's callback executes "/close", 'sess' may be invalid */
 	if (is_session (sess))
-		display_event (pntevts[index], sess, te[index].num_args, word);
+		display_event (sess, index, word);
 }
 
 int
@@ -1715,7 +1686,7 @@ sound_play (const char *file)
 	} else
 	{
 		snprintf (buf, sizeof (buf), _("Cannot read sound file:\n%s"), wavfile);
-		fe_message (buf, FALSE);
+		fe_message (buf, FE_MSG_ERROR);
 	}
 
 	g_free (file_fs);
