@@ -19,6 +19,8 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
 #include "connect-dialog.h"
 #include "navigation-tree.h"
 #include "gui.h"
@@ -98,8 +100,8 @@ connect_dialog_init (ConnectDialog *dialog)
 	dialog->toplevel = NULL;
 
 	xml = NULL;
-	if (g_file_test ("connect-dialog.glade", G_FILE_TEST_EXISTS))
-		xml = glade_xml_new ("connect-dialog.glade", "toplevel", NULL);
+	if (g_file_test ("../../data/connect-dialog.glade", G_FILE_TEST_EXISTS))
+		xml = glade_xml_new ("../../data/connect-dialog.glade", "toplevel", NULL);
 	if (!xml)
 		xml = glade_xml_new (XCHATSHAREDIR "/connect-dialog.glade", "toplevel", NULL);
 	if (!xml)
@@ -141,6 +143,7 @@ connect_dialog_init (ConnectDialog *dialog)
 
 	gtk_window_set_default_size (GTK_WINDOW (dialog), 320, 240);
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Connect"));
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, FALSE);
 	g_signal_connect (G_OBJECT (dialog), "response", G_CALLBACK (dialog_response), NULL);
 }
@@ -168,7 +171,7 @@ connect_dialog_get_type (void)
 }
 
 ConnectDialog *
-connect_dialog_new ()
+connect_dialog_new (void)
 {
 	ConnectDialog *dialog = g_object_new (connect_dialog_get_type(), 0);
 	if (dialog->toplevel == NULL) {
