@@ -20,15 +20,14 @@
  */
 
 #include <glade/glade.h>
-#include <gnome.h>
-#include "../fe-gtk/xtext.h"
-#include "../common/xchat.h"
+#include <gtk/gtkactiongroup.h>
+#include <gtk/gtkuimanager.h>
 #include "userlist.h"
 #include "navigation-tree.h"
 #include "dcc-window.h"
+#include "xtext.h"
 
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtkuimanager.h>
+#include "../common/xchat.h"
 
 #ifndef XCHAT_GNOME_GUI_H
 #define XCHAT_GNOME_GUI_H
@@ -37,21 +36,24 @@ typedef struct
 {
 	GladeXML *xml;
 	GladeXML *setup_xml;
-	GnomeApp *main_window;
 	GtkActionGroup *action_group;
 	GtkUIManager *manager;
-	GnomeAbout *about;
-	GtkXText *xtext;
+	GtkWidget *about;
 	DccWindow *dcc;
-#if (GTK_CHECK_VERSION(2,5,0))
 	GtkExpander *topic_expander;
-#endif
-	GtkWidget *topic_label;
-	GtkWidget *text_entry;
+	GtkWidget *userlist_window;
+	GtkWidget *userlist;
+	GtkWidget *userlist_toggle;
 	session *current_session;
-	GnomeDruid *setup_druid;
 	gboolean quit;
 
+	GtkWidget *conversation_panel;
+	GtkWidget *find_bar;
+	GtkWidget *main_window;
+	GtkWidget *status_bar;
+	GtkWidget *text_entry;
+	GtkWidget *topic_label;
+
 	NavModel *tree_model;
 	NavTree *server_tree;
 } XChatGUI;
@@ -59,8 +61,8 @@ typedef struct
 extern XChatGUI gui;
 extern Userlist *u;
 
-gboolean initialize_gui_1 ();
-gboolean initialize_gui_2 ();
+gboolean initialize_gui_1 (void);
+gboolean initialize_gui_2 (void);
 int xtext_get_stamp_str (time_t tim, char **ret);
 
 #endif
