@@ -19,92 +19,96 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include <gconf/gconf-client.h>
 #include <gtk/gtk.h>
 #include "navigation-tree.h"
 #include "userlist-gui.h"
 #include "userlist.h"
-#include "textgui.h"
 #include "pixmaps.h"
 #include "palette.h"
 #include "channel-list.h"
 #include "main-window.h"
+
+#include "conversation-panel.h"
+#include "find-bar.h"
+#include "status-bar.h"
+#include "text-entry.h"
+#include "topic-label.h"
+
 #include "util.h"
 #include "../common/xchat.h"
 #include "../common/fe.h"
 #include "../common/servlist.h"
 #include "../common/plugin.h"
 
-#ifdef HAVE_LIBSEXY
-#include <libsexy/sexy-url-label.h>
-#endif
-
 /***** NavTree *****/
-static void navigation_tree_init (NavTree * navtree);
-static void navigation_tree_class_init (NavTreeClass * klass);
-static void navigation_tree_dispose (GObject * object);
-static void navigation_tree_finalize (GObject * object);
-static session* navigation_tree_get_selected_session (void);
-static void navigation_tree_update_refs (NavTree* navtree);
+static void      navigation_tree_init (NavTree *navtree);
+static void      navigation_tree_class_init (NavTreeClass *klass);
+static void      navigation_tree_dispose (GObject *object);
+static void      navigation_tree_finalize (GObject *object);
+static session  *navigation_tree_get_selected_session (void);
+static void      navigation_tree_update_refs (NavTree *navtree);
 /* Context menus. */
-static void navigation_context (GtkWidget * treeview, session * selected);
-static void server_context (GtkWidget * treeview, session * selected);
-static void channel_context (GtkWidget * treeview, session * selected);
-static void dialog_context (GtkWidget * treeview, session * selected);
-static gboolean channel_is_autojoin (ircnet * network, gchar *chan);
+static void      navigation_context (GtkWidget *treeview, session *selected);
+static void      server_context (GtkWidget *treeview, session *selected);
+static void      channel_context (GtkWidget *treeview, session *selected);
+static void      dialog_context (GtkWidget *treeview, session *selected);
+static gboolean  channel_is_autojoin (ircnet *network, gchar *chan);
 /* Callbacks. */
-static gboolean click (GtkWidget * widget, GdkEventButton * event, gpointer user_data);
-static gboolean declick (GtkWidget * widget, GdkEventButton * event, gpointer user_data);
-static void navigation_selection_changed (GtkTreeSelection * treeselection, gpointer user_data);
-static void row_expanded (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, gpointer user_data);
-static void row_collapsed (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, gpointer user_data);
-static void on_server_information (GtkAction * action, gpointer data);
-static void on_server_reconnect (GtkAction * action, gpointer data);
-static void on_server_disconnect (GtkAction * action, gpointer data);
-static void on_server_channel_list (GtkAction * action, gpointer data);
-static void on_save (GtkAction * action, gpointer data);
-static void on_close (GtkAction * action, gpointer data);
-static void on_find (GtkAction * action, gpointer data);
-static void on_find_again (GtkAction * action, gpointer data);
-static void on_clear (GtkAction * action, gpointer data);
-static void on_channel_leave (GtkAction * action, gpointer data);
-static void on_channel_bans (GtkAction * action, gpointer data);
-static void on_server_autoconnect (GtkAction * action, gpointer data);
-static void on_channel_autojoin (GtkAction * action, gpointer data);
+static gboolean  click (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+static gboolean  declick (GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+static void      navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data);
+static void      row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
+static void      row_collapsed (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data);
+static void      on_server_information (GtkAction *action, gpointer data);
+static void      on_server_reconnect (GtkAction *action, gpointer data);
+static void      on_server_disconnect (GtkAction *action, gpointer data);
+static void      on_server_channel_list (GtkAction *action, gpointer data);
+static void      on_save (GtkAction *action, gpointer data);
+static void      on_close (GtkAction *action, gpointer data);
+static void      on_find (GtkAction *action, gpointer data);
+static void      on_clear (GtkAction *action, gpointer data);
+static void      on_channel_leave (GtkAction *action, gpointer data);
+static void      on_channel_bans (GtkAction *action, gpointer data);
+static void      on_server_autoconnect (GtkAction *action, gpointer data);
+static void      on_channel_autojoin (GtkAction *action, gpointer data);
 
 static GtkActionEntry action_entries[] = {
 	/* Server context menu */
-	{"ServerInformation", GTK_STOCK_DIALOG_INFO,    _("_Information"),     "", NULL, G_CALLBACK (on_server_information)},
-	{"ServerReconnect",   GTK_STOCK_REFRESH,        _("_Reconnect"),       "", NULL, G_CALLBACK (on_server_reconnect)},
-	{"ServerDisconnect",  GTK_STOCK_STOP,           _("_Disconnect"),      "", NULL, G_CALLBACK (on_server_disconnect)},
-	{"ServerClose",       GTK_STOCK_CLOSE,          _("_Close"),           "", NULL, G_CALLBACK (on_close)},
-	{"ServerChannels",    GTK_STOCK_INDEX,          _("_Channels..."),     "", NULL, G_CALLBACK (on_server_channel_list)},
+	{"ServerInformation", GTK_STOCK_DIALOG_INFO,    N_("_Information"),     "", NULL, G_CALLBACK (on_server_information)},
+	{"ServerReconnect",   GTK_STOCK_REFRESH,        N_("_Reconnect"),       "", NULL, G_CALLBACK (on_server_reconnect)},
+	{"ServerDisconnect",  GTK_STOCK_STOP,           N_("_Disconnect"),      "", NULL, G_CALLBACK (on_server_disconnect)},
+	{"ServerClose",       GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
+	{"ServerChannels",    GTK_STOCK_INDEX,          N_("_Channels..."),     "", NULL, G_CALLBACK (on_server_channel_list)},
 
 	/* Channel context menu */
-	{"ChannelSave",       GTK_STOCK_SAVE,           _("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
-	{"ChannelLeave",      GTK_STOCK_QUIT,           _("_Leave"),           "", NULL, G_CALLBACK (on_channel_leave)},
-	{"ChannelClose",      GTK_STOCK_CLOSE,          _("_Close"),           "", NULL, G_CALLBACK (on_close)},
-	{"ChannelFind",       GTK_STOCK_FIND,           _("_Find..."),         "", NULL, G_CALLBACK (on_find)},
-	{"ChannelFindAgain",  GTK_STOCK_FIND,           _("Find Ne_xt"),       "", NULL, G_CALLBACK (on_find_again)},
-	{"ChannelClear",      GTK_STOCK_CLEAR,          _("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
-	{"ChannelBans",       GTK_STOCK_DIALOG_WARNING, _("_Bans..."),         "", NULL, G_CALLBACK (on_channel_bans)},
+	{"ChannelSave",       GTK_STOCK_SAVE,           N_("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
+	{"ChannelLeave",      GTK_STOCK_QUIT,           N_("_Leave"),           "", NULL, G_CALLBACK (on_channel_leave)},
+	{"ChannelClose",      GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
+	{"ChannelFind",       GTK_STOCK_FIND,           N_("_Find..."),         "", NULL, G_CALLBACK (on_find)},
+	{"ChannelClear",      GTK_STOCK_CLEAR,          N_("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
+	{"ChannelBans",       GTK_STOCK_DIALOG_WARNING, N_("_Bans..."),         "", NULL, G_CALLBACK (on_channel_bans)},
 
 	/* Dialog context menu */
-	{"DialogSave",        GTK_STOCK_SAVE,           _("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
-	{"DialogClose",       GTK_STOCK_CLOSE,          _("_Close"),           "", NULL, G_CALLBACK (on_close)},
-	{"DialogFind",        GTK_STOCK_FIND,           _("_Find..."),         "", NULL, G_CALLBACK (on_find)},
-	{"DialogFindAgain",   GTK_STOCK_FIND,           _("Find Ne_xt"),       "", NULL, G_CALLBACK (on_find_again)},
-	{"DialogClear",       GTK_STOCK_CLEAR,          _("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
+	{"DialogSave",        GTK_STOCK_SAVE,           N_("_Save Transcript"), "", NULL, G_CALLBACK (on_save)},
+	{"DialogClose",       GTK_STOCK_CLOSE,          N_("_Close"),           "", NULL, G_CALLBACK (on_close)},
+	{"DialogFind",        GTK_STOCK_FIND,           N_("_Find..."),         "", NULL, G_CALLBACK (on_find)},
+	{"DialogClear",       GTK_STOCK_CLEAR,          N_("C_lear"),           "", NULL, G_CALLBACK (on_clear)},
 };
 
 static GtkToggleActionEntry toggle_action_entries[] = {
 	/* Server context menu */
-	{"ServerAutoConnect", NULL,          _("_Auto-connect"),    "", NULL, G_CALLBACK (on_server_autoconnect), FALSE},
+	{"ServerAutoConnect", NULL, N_("_Auto-connect"), "", NULL, G_CALLBACK (on_server_autoconnect), FALSE},
 
 	/* Channel context menu */
-	{"ChannelAutoJoin", NULL,          _("_Auto-join"),    "", NULL, G_CALLBACK (on_channel_autojoin), FALSE},
+	{"ChannelAutoJoin",   NULL, N_("_Auto-join"),    "", NULL, G_CALLBACK (on_channel_autojoin), FALSE},
 };
 
+static GtkActionGroup *action_group;
+
 GType
 navigation_tree_get_type (void)
 {
@@ -132,7 +136,7 @@ navigation_tree_get_type (void)
 }
 
 static void
-navigation_tree_init (NavTree * navtree)
+navigation_tree_init (NavTree *navtree)
 {
 	GtkCellRenderer *icon_renderer, *text_renderer;
 	GtkTreeViewColumn *column;
@@ -143,11 +147,12 @@ navigation_tree_init (NavTree * navtree)
 	navtree->model = NULL;
 	navtree->selection_changed_id = 0;
 
-	gui.action_group = gtk_action_group_new ("MenuAction");
-	gtk_action_group_add_actions (gui.action_group, action_entries, G_N_ELEMENTS (action_entries), NULL);
-	gtk_action_group_add_toggle_actions (gui.action_group, toggle_action_entries, G_N_ELEMENTS (toggle_action_entries), NULL);
-	gtk_ui_manager_insert_action_group (gui.manager, gui.action_group, 0);
-	g_object_unref (gui.action_group);
+	action_group = gtk_action_group_new ("NavigationContext");
+	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+	gtk_action_group_add_actions (action_group, action_entries, G_N_ELEMENTS (action_entries), NULL);
+	gtk_action_group_add_toggle_actions (action_group, toggle_action_entries, G_N_ELEMENTS (toggle_action_entries), NULL);
+	gtk_ui_manager_insert_action_group (gui.manager, action_group, 0);
+	g_object_unref (action_group);
 
 	/* This sets up all our columns. */
 	column = gtk_tree_view_column_new ();
@@ -179,7 +184,7 @@ navigation_tree_init (NavTree * navtree)
 }
 
 static void
-navigation_tree_class_init (NavTreeClass * klass)
+navigation_tree_class_init (NavTreeClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *) klass;
 
@@ -188,7 +193,7 @@ navigation_tree_class_init (NavTreeClass * klass)
 }
 
 static void
-navigation_tree_dispose (GObject * object)
+navigation_tree_dispose (GObject *object)
 {
 	NavTree *navtree = (NavTree *) object;
 	if (navtree->model) {
@@ -198,7 +203,7 @@ navigation_tree_dispose (GObject * object)
 }
 
 static void
-navigation_tree_finalize (GObject * object)
+navigation_tree_finalize (GObject *object)
 {
 	NavTree *navtree = (NavTree *) object;
 	gtk_tree_path_free (navtree->current_path);
@@ -207,7 +212,7 @@ navigation_tree_finalize (GObject * object)
 
 /* New NavTree. */
 NavTree *
-navigation_tree_new (NavModel * model)
+navigation_tree_new (NavModel *model)
 {
 	NavTree *new_tree;
 
@@ -223,7 +228,7 @@ navigation_tree_new (NavModel * model)
 
 /* Add/remove server/channel functions. */
 void
-navigation_tree_create_new_network_entry (NavTree * navtree, struct session *sess)
+navigation_tree_create_new_network_entry (NavTree *navtree, struct session *sess)
 {
 	GtkTreeIter *iter;
 	GtkWidget *menuitem, *button;
@@ -238,6 +243,9 @@ navigation_tree_create_new_network_entry (NavTree * navtree, struct session *ses
 	 */
 	iter = navigation_model_get_sorted_iter (navtree->model, gui.current_session);
 	if (iter) {
+		if (navtree->current_path)
+			gtk_tree_path_free (navtree->current_path);
+
 		navtree->current_path = gtk_tree_model_get_path (GTK_TREE_MODEL (navtree->model->sorted), iter);
 		gtk_tree_iter_free (iter);
 	}
@@ -259,13 +267,12 @@ navigation_tree_create_new_network_entry (NavTree * navtree, struct session *ses
 }
 
 void
-navigation_tree_create_new_channel_entry (NavTree * navtree, struct session *sess)
+navigation_tree_create_new_channel_entry (NavTree *navtree, struct session *sess)
 {
 	GtkTreeIter *iter;
 	GtkWidget *menuitem, *button;
 	GtkTreeView *treeview;
 	ircnet *net;
-	session_gui *tgui;
 
 	navigation_model_add_new_channel (navtree->model, sess);
 
@@ -279,6 +286,9 @@ navigation_tree_create_new_channel_entry (NavTree * navtree, struct session *ses
 	 */
 	iter = navigation_model_get_sorted_iter (navtree->model, gui.current_session);
 	if (iter) {
+		if (navtree->current_path)
+			gtk_tree_path_free (navtree->current_path);
+
 		navtree->current_path = gtk_tree_model_get_path (GTK_TREE_MODEL (navtree->model->sorted), iter);
 		gtk_tree_iter_free (iter);
 	}
@@ -290,12 +300,7 @@ navigation_tree_create_new_channel_entry (NavTree * navtree, struct session *ses
 	treeview = GTK_TREE_VIEW (glade_xml_get_widget (gui.xml, "userlist"));
 	gtk_tree_view_set_model (treeview, GTK_TREE_MODEL (userlist_get_store (u, sess)));
 
-	tgui = (session_gui *) sess->gui;
-#ifdef HAVE_LIBSEXY
-	sexy_url_label_set_markup (SEXY_URL_LABEL (gui.topic_label), tgui->url_topic);
-#else
-	gtk_label_set_text (GTK_LABEL (gui.topic_label), tgui->topic);
-#endif
+	topic_label_set_current (TOPIC_LABEL (gui.topic_label), sess);
 	net = sess->server->network;
 	if (net == NULL)
 		rename_main_window (NULL, sess->channel);
@@ -313,7 +318,7 @@ navigation_tree_create_new_channel_entry (NavTree * navtree, struct session *ses
 }
 
 void
-navigation_tree_remove_channel (NavTree * navtree, struct session *sess)
+navigation_tree_remove_channel (NavTree *navtree, struct session *sess)
 {
 	GtkTreePath *path = gtk_tree_path_copy (navtree->current_path);
 	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
@@ -352,7 +357,7 @@ navigation_tree_remove_channel (NavTree * navtree, struct session *sess)
 }
 
 void
-navigation_tree_remove_server (NavTree * navtree, struct session *sess)
+navigation_tree_remove_server (NavTree *navtree, struct session *sess)
 {
 	GtkTreePath *path = gtk_tree_path_copy (navtree->current_path);
 	GtkTreeSelection *select = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
@@ -395,7 +400,7 @@ navigation_tree_remove_server (NavTree * navtree, struct session *sess)
 }
 
 void
-navigation_tree_server_rm_chans (NavTree * navtree, GtkTreeIter * parent)
+navigation_tree_server_rm_chans (NavTree *navtree, GtkTreeIter * parent)
 {
 	GtkTreeModel *sorted = gtk_tree_view_get_model (GTK_TREE_VIEW (navtree));
 	GtkTreeModel *store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (sorted));
@@ -406,14 +411,16 @@ navigation_tree_server_rm_chans (NavTree * navtree, GtkTreeIter * parent)
 		do {
 			gtk_tree_model_get (store, &child, 2, &s, -1);
 			fe_close_window (s);
-			text_gui_remove_text_buffer (s);
+			conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
+			topic_label_remove_session        (TOPIC_LABEL        (gui.topic_label),        s);
+			text_entry_remove_session         (TEXT_ENTRY         (gui.text_entry),         s);
 		} while (gtk_tree_model_iter_next (store, &child));
 	}
 }
 
 /* Channel/server selection functions. */
 void
-navigation_tree_select_nth_channel (NavTree * navtree, gint chan_num)
+navigation_tree_select_nth_channel (NavTree *navtree, gint chan_num)
 {
 	GtkTreeView *view;
 	GtkTreeModel *model;
@@ -465,7 +472,7 @@ navigation_tree_select_nth_channel (NavTree * navtree, gint chan_num)
 }
 
 void
-navigation_tree_select_session (NavTree * navtree, struct session *sess)
+navigation_tree_select_session (NavTree *navtree, struct session *sess)
 {
 	GtkTreeIter *iter = navigation_model_get_sorted_iter (navtree->model, sess);
 
@@ -487,16 +494,16 @@ navigation_tree_select_session (NavTree * navtree, struct session *sess)
 }
 
 void
-navigation_tree_select_next_channel (NavTree * navtree)
+navigation_tree_select_next_channel (NavTree *navtree)
 {
-	GtkTreeIter		iter;
-	GtkTreeModel*		model;
-	GtkTreeSelection*	selection;
-	GtkTreePath*		path;
-	gchar*			path_string;
-	gint			depth;
-	gint			server_index = 0;
-	gint			channel_index = 0;
+	GtkTreeIter       iter;
+	GtkTreeModel     *model;
+	GtkTreeSelection *selection;
+	GtkTreePath      *path;
+	gchar            *path_string;
+	gint              depth;
+	gint              server_index = 0;
+	gint              channel_index = 0;
 
 
 	/* Get the GtkSelection from the GtkTreeView. */
@@ -515,11 +522,11 @@ navigation_tree_select_next_channel (NavTree * navtree)
 
 	/* If a channel is currently selected... */
 	if (depth == 2) {
-		gint	channels = 0;
-		gchar**	split = g_strsplit (path_string, ":", 2);
+		gint    channels = 0;
+		gchar** split = g_strsplit (path_string, ":", 2);
 
-		server_index = atoi (split[0]);
-		channel_index = atoi (split[1]);
+		server_index  = g_strtod (split[0], NULL);
+		channel_index = g_strtod (split[1], NULL);
 
 		g_strfreev (split);
 
@@ -552,7 +559,7 @@ navigation_tree_select_next_channel (NavTree * navtree)
 			}
 		}
 	} else {
-		server_index = atoi (path_string);
+		server_index = g_strtod (path_string, NULL);
 	}
 
 	if (depth == 1) {
@@ -604,13 +611,13 @@ navigation_tree_select_next_channel (NavTree * navtree)
 }
 
 void
-navigation_tree_select_prev_channel (NavTree * navtree)
+navigation_tree_select_prev_channel (NavTree *navtree)
 {
-	GtkTreeModel*		model;
-	GtkTreeSelection*	selection;
-	GtkTreeIter		iter;
-	GtkTreePath*		path;
-	gint			depth;
+	GtkTreeModel     *model;
+	GtkTreeSelection *selection;
+	GtkTreeIter       iter;
+	GtkTreePath      *path;
+	gint              depth;
 
 	/* Get the GtkTreeSelection. */
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
@@ -689,11 +696,11 @@ navigation_tree_select_prev_channel (NavTree * navtree)
 }
 
 void
-navigation_tree_select_next_network (NavTree * navtree)
+navigation_tree_select_next_network (NavTree *navtree)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *model;
-	GtkTreePath *path;
+	GtkTreeIter      iter;
+	GtkTreeModel     *model;
+	GtkTreePath      *path;
 	GtkTreeSelection *selection;
 
 	/* Get the GtkTreeSelection. */
@@ -735,12 +742,12 @@ navigation_tree_select_next_network (NavTree * navtree)
 }
 
 void
-navigation_tree_select_prev_network (NavTree * navtree)
+navigation_tree_select_prev_network (NavTree *navtree)
 {
-	GtkTreeIter iter;
-	GtkTreeModel *model;
+	GtkTreeIter       iter;
+	GtkTreeModel     *model;
 	GtkTreeSelection *selection;
-	GtkTreePath *path;
+	GtkTreePath      *path;
 
 	/* Get our tree view and selection. */
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (navtree));
@@ -801,7 +808,7 @@ navigation_tree_get_selected_session (void)
 }
 
 static void
-navigation_tree_update_refs (NavTree* navtree)
+navigation_tree_update_refs (NavTree *navtree)
 {
 	GtkTreeModel*	model = GTK_TREE_MODEL (navtree->model->sorted);
 	GtkTreeIter	iter;
@@ -866,7 +873,7 @@ navigation_tree_update_refs (NavTree* navtree)
 
 /* Misc. Functions. */
 static gboolean
-navigation_tree_set_channel_name_iterate (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
+navigation_tree_set_channel_name_iterate (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
 	/* Iterator function for set_channel_name. */
 	gpointer s;
@@ -880,7 +887,7 @@ navigation_tree_set_channel_name_iterate (GtkTreeModel * model, GtkTreePath * pa
 }
 
 void
-navigation_tree_set_channel_name (NavTree * navtree, struct session *sess)
+navigation_tree_set_channel_name (NavTree *navtree, struct session *sess)
 {
 	GtkTreeModel *store, *model;
 
@@ -891,7 +898,7 @@ navigation_tree_set_channel_name (NavTree * navtree, struct session *sess)
 }
 
 gboolean
-navigation_tree_server_is_connected (NavTree * navtree, gchar * name)
+navigation_tree_server_is_connected (NavTree *navtree, gchar *name)
 {
 	GtkTreeModel *model, *store;
 	GtkTreeIter iter;
@@ -918,7 +925,7 @@ navigation_tree_server_is_connected (NavTree * navtree, gchar * name)
 
 /***** Context Menus *****/
 static void
-navigation_context (GtkWidget * treeview, session * selected)
+navigation_context (GtkWidget *treeview, session *selected)
 {
 	switch (selected->type) {
 	case SESS_SERVER:
@@ -934,7 +941,7 @@ navigation_context (GtkWidget * treeview, session * selected)
 }
 
 static void
-server_context (GtkWidget * treeview, session * selected)
+server_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
 	GtkAction *action;
@@ -942,18 +949,28 @@ server_context (GtkWidget * treeview, session * selected)
 	menu = gtk_ui_manager_get_widget (gui.manager, "/ServerPopup");
 	g_return_if_fail (menu != NULL);
 
-	/* check if the network is in the auto-connect list */
-	action = gtk_action_group_get_action (gui.action_group, "ServerAutoConnect");
-	if (((ircnet *)selected->server->network)->flags & FLAG_AUTO_CONNECT)
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-	else
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+	if (((ircnet *)selected->server->network) == NULL) {
+		/* disable the auto-connect action.
+		 * FIXME: it would be really nice to be able to configure this server
+		 * as part of a network here.
+		 */
+		action = gtk_action_group_get_action (action_group, "ServerAutoConnect");
+		gtk_action_set_sensitive (action, FALSE);
+	} else if (selected != NULL) {
+		/* check if the network is in the auto-connect list */
+		action = gtk_action_group_get_action (action_group, "ServerAutoConnect");
+		gtk_action_set_sensitive (action, TRUE);
+		if (((ircnet *)selected->server->network)->flags & FLAG_AUTO_CONNECT)
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+		else
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+	}
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 }
 
 static void
-channel_context (GtkWidget * treeview, session * selected)
+channel_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
 	GtkAction *action;
@@ -963,18 +980,24 @@ channel_context (GtkWidget * treeview, session * selected)
 	g_return_if_fail (menu != NULL);
 
 	/* check if the channel is in the auto-join list */
-	action = gtk_action_group_get_action (gui.action_group, "ChannelAutoJoin");
+	action = gtk_action_group_get_action (action_group, "ChannelAutoJoin");
 	network = selected->server->network;
-	if (channel_is_autojoin (network, selected->channel))
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-	else
-		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+
+	if (network == NULL) {
+		gtk_action_set_sensitive (action, FALSE);
+	} else {
+		gtk_action_set_sensitive (action, TRUE);
+		if (channel_is_autojoin (network, selected->channel))
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
+		else
+			gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), FALSE);
+	}
 
 	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 }
 
 static void
-dialog_context (GtkWidget * treeview, session * selected)
+dialog_context (GtkWidget *treeview, session *selected)
 {
 	GtkWidget *menu;
 
@@ -986,7 +1009,7 @@ dialog_context (GtkWidget * treeview, session * selected)
 
 /***** Callbacks *****/
 static gboolean
-click (GtkWidget * treeview, GdkEventButton * event, gpointer data)
+click (GtkWidget *treeview, GdkEventButton *event, gpointer data)
 {
 	GtkTreePath *path;
 	GtkTreeSelection *select;
@@ -1019,7 +1042,7 @@ click (GtkWidget * treeview, GdkEventButton * event, gpointer data)
 }
 
 static gboolean
-declick (GtkWidget * treeview, GdkEventButton * e, gpointer data)
+declick (GtkWidget *treeview, GdkEventButton *e, gpointer data)
 {
 	gtk_widget_grab_focus (gui.text_entry);
 	gtk_editable_set_position (GTK_EDITABLE (gui.text_entry), -1);
@@ -1028,7 +1051,7 @@ declick (GtkWidget * treeview, GdkEventButton * e, gpointer data)
 }
 
 static void
-navigation_selection_changed (GtkTreeSelection * treeselection, gpointer user_data)
+navigation_selection_changed (GtkTreeSelection *treeselection, gpointer user_data)
 {
 	GtkTreeIter iter, newiter;
 	GtkTreeModel *model, *store;
@@ -1036,26 +1059,24 @@ navigation_selection_changed (GtkTreeSelection * treeselection, gpointer user_da
 	GtkWidget *button;
 	gpointer *s;
 	session *sess;
-	session_gui *tgui;
+
+	if (gui.server_tree == NULL)
+		return;
 
 	treeview = GTK_TREE_VIEW (glade_xml_get_widget (gui.xml, "userlist"));
 
 	if (gui.server_tree->current_path != NULL)
 		navigation_model_path_deref (gui.server_tree->model, gui.server_tree->current_path);
 
+	/* If find bar is open, hide it */
+	find_bar_close (FIND_BAR (gui.find_bar));
+
 	/* XXX: This sets model to be the GtkTreeModelSort used by the NavTree, it is
 	 *      not a GtkTreeModel. The iter is for that ModelSort.
 	 */
 	if (gtk_tree_selection_get_selected (treeselection, &model, &iter) && gui.current_session) {
 		GtkWidget *menuitem;
 
-		/* back up existing entry */
-		tgui = (session_gui *) gui.current_session->gui;
-		if (tgui) {
-			g_free (tgui->entry);
-			tgui->entry = g_strdup (gtk_entry_get_text (GTK_ENTRY (gui.text_entry)));
-		}
-
 		/* Update current_path. */
 		if (gui.server_tree->current_path) {
 			gtk_tree_path_free (gui.server_tree->current_path);
@@ -1091,44 +1112,33 @@ navigation_selection_changed (GtkTreeSelection * treeselection, gpointer user_da
 			rename_main_window (net->name, sess->channel);
 		}
 
+		/* Set widget sensitivity based on type of the current "tab" */
 		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionChangeTopicItem");
 		if (menuitem == NULL)
 			g_warning ("can't disable topic change menu item");
 		else
 			gtk_widget_set_sensitive (menuitem, sess->type == SESS_CHANNEL);
 		button = glade_xml_get_widget (gui.xml, "close discussion");
-		if (sess->type != SESS_SERVER)
-			gtk_widget_set_sensitive (button, TRUE);
-		else
-			gtk_widget_set_sensitive (button, FALSE);
+		gtk_widget_set_sensitive (button, sess->type != SESS_SERVER);
+
+		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionUsersItem");
+		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
+		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionLeaveItem");
+		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
+		menuitem = gtk_ui_manager_get_widget (gui.manager, "/ui/menubar/DiscussionMenu/DiscussionCloseItem");
+		gtk_widget_set_sensitive (menuitem, sess->type != SESS_SERVER);
+		gtk_widget_set_sensitive (gui.userlist_toggle, sess->type != SESS_SERVER);
+
 
 		/* remove any icon that exists */
 		store = gtk_tree_model_sort_get_model (GTK_TREE_MODEL_SORT (model));
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model), &newiter, &iter);
 		gtk_tree_store_set (GTK_TREE_STORE (store), &newiter, 0, NULL, 3, 0, -1);
 
-		/* Set tgui to the gui of the new session. */
-		tgui = (session_gui *) sess->gui;
-		if (tgui) {
-			/* Set the topic. */
-#ifdef HAVE_LIBSEXY
-			sexy_url_label_set_markup (SEXY_URL_LABEL (gui.topic_label), tgui->url_topic);
-#else
-			gtk_label_set_text (GTK_LABEL (gui.topic_label), tgui->topic);
-#endif
-
-			/* Show the xtext buffer for the session. */
-			gtk_xtext_buffer_show (gui.xtext, tgui->buffer, TRUE);
-
-			/* Set the text entry field to whatever is in the text entry of this session. */
-			gtk_entry_set_text (GTK_ENTRY (gui.text_entry), tgui->entry);
-			gtk_entry_set_position (GTK_ENTRY (gui.text_entry), -1);
-		} else {
-			/* If there's no gui for the new session make sure the entry is empty
-			 * and then return.
-			 */
-			gtk_entry_set_text (GTK_ENTRY (gui.text_entry), "");
-		}
+		conversation_panel_set_current (CONVERSATION_PANEL (gui.conversation_panel), sess);
+		topic_label_set_current        (TOPIC_LABEL        (gui.topic_label),        sess);
+		text_entry_set_current         (TEXT_ENTRY         (gui.text_entry),         sess);
+		status_bar_set_current         (STATUS_BAR         (gui.status_bar),         sess->server);
 
 		/* Emit "focus tab" event */
 		plugin_emit_dummy_print (sess, "Focus Tab");
@@ -1136,7 +1146,7 @@ navigation_selection_changed (GtkTreeSelection * treeselection, gpointer user_da
 }
 
 static void
-row_expanded (GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, gpointer user_data)
+row_expanded (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *path, gpointer user_data)
 {
 	/* If we had something selected before the row was collapsed make sure it gets selected. */
 	if (NAVTREE (treeview)->current_path && gtk_tree_path_is_ancestor (path, NAVTREE (treeview)->current_path)) {
@@ -1189,15 +1199,15 @@ navigation_model_get_type (void)
 }
 
 static void
-navigation_model_init (NavModel * navmodel)
+navigation_model_init (NavModel *navmodel)
 {
-	navmodel->store = gtk_tree_store_new (7, GDK_TYPE_PIXBUF,	/* status image */
-						 G_TYPE_STRING,		/* name */
-						 G_TYPE_POINTER,	/* session pointer */
-						 G_TYPE_INT,		/* status # (for tracking highest state) */
-						 GDK_TYPE_COLOR,	/* status color (disconnected, etc) */
-						 G_TYPE_INT,		/* reference count */
-						 G_TYPE_BOOLEAN);	/* connected */
+	navmodel->store = gtk_tree_store_new (7, GDK_TYPE_PIXBUF, /* status image */
+	                                         G_TYPE_STRING,   /* name */
+	                                         G_TYPE_POINTER,  /* session pointer */
+	                                         G_TYPE_INT,      /* status # (for tracking highest state) */
+	                                         GDK_TYPE_COLOR,  /* status color (disconnected, etc) */
+	                                         G_TYPE_INT,      /* reference count */
+	                                         G_TYPE_BOOLEAN); /* connected */
 	navmodel->sorted = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (navmodel->store));
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (navmodel->sorted), 1, GTK_SORT_ASCENDING);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (navmodel->sorted), 1, gtk_tree_iter_sort_func_nocase, NULL, NULL);
@@ -1209,7 +1219,7 @@ navigation_model_init (NavModel * navmodel)
 }
 
 static void
-navigation_model_class_init (NavModelClass * klass)
+navigation_model_class_init (NavModelClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass *) klass;
 	object_class->dispose = navigation_model_dispose;
@@ -1217,13 +1227,13 @@ navigation_model_class_init (NavModelClass * klass)
 }
 
 static void
-destroy_rowref (session * sess, GtkTreeRowReference * rowref, gpointer data)
+destroy_rowref (session *sess, GtkTreeRowReference *rowref, gpointer data)
 {
 	gtk_tree_row_reference_free (rowref);
 }
 
 static void
-navigation_model_dispose (GObject * object)
+navigation_model_dispose (GObject *object)
 {
 	NavModel *navmodel = NAVMODEL (object);
 	g_object_unref ((gpointer) navmodel->store);
@@ -1232,7 +1242,7 @@ navigation_model_dispose (GObject * object)
 }
 
 static void
-navigation_model_finalize (GObject * object)
+navigation_model_finalize (GObject *object)
 {
 	NavModel *model = (NavModel *) object;
 	g_hash_table_destroy (model->session_rows);
@@ -1246,11 +1256,11 @@ navigation_model_new ()
 }
 
 void
-navigation_model_add_new_network (NavModel * model, struct session *sess)
+navigation_model_add_new_network (NavModel *model, struct session *sess)
 {
-	GtkTreeIter		iter;
-	GtkTreePath*		path;
-	GtkTreeRowReference*	rowref;
+	GtkTreeIter          iter;
+	GtkTreePath         *path;
+	GtkTreeRowReference *rowref;
 
 	gtk_tree_store_append (model->store, &iter, NULL);
 
@@ -1270,7 +1280,7 @@ navigation_model_add_new_network (NavModel * model, struct session *sess)
 }
 
 static gboolean
-navigation_model_create_new_channel_entry_iterate (GtkTreeModel * model, GtkTreePath * path, GtkTreeIter * iter, struct session *data)
+navigation_model_create_new_channel_entry_iterate (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, struct session *data)
 {
 	struct session *s;
 	gtk_tree_model_get (model, iter, 2, &s, -1);
@@ -1302,16 +1312,16 @@ navigation_model_create_new_channel_entry_iterate (GtkTreeModel * model, GtkTree
 }
 
 void
-navigation_model_add_new_channel (NavModel * model, struct session *sess)
+navigation_model_add_new_channel (NavModel *model, struct session *sess)
 {
 	gtk_tree_model_foreach (GTK_TREE_MODEL (model->store), (GtkTreeModelForeachFunc) navigation_model_create_new_channel_entry_iterate, (gpointer) sess);
 }
 
 void
-navigation_model_remove (NavModel * model, struct session *sess)
+navigation_model_remove (NavModel *model, struct session *sess)
 {
-	GtkTreeIter*	iter = navigation_model_get_unsorted_iter (model, sess);
-	GtkTreePath*	path = gtk_tree_model_get_path (GTK_TREE_MODEL (model->store), iter);
+	GtkTreeIter *iter = navigation_model_get_unsorted_iter (model, sess);
+	GtkTreePath *path = gtk_tree_model_get_path (GTK_TREE_MODEL (model->store), iter);
 
 	if (gtk_tree_path_get_depth (path) == 1)
 		model->servers--;
@@ -1323,7 +1333,7 @@ navigation_model_remove (NavModel * model, struct session *sess)
 }
 
 GtkTreeIter *
-navigation_model_get_sorted_iter (NavModel * model, struct session *sess)
+navigation_model_get_sorted_iter (NavModel *model, struct session *sess)
 {
 	GtkTreeIter *iter = NULL;
 	GtkTreePath *path, *sorted;
@@ -1534,6 +1544,7 @@ on_server_channel_list (GtkAction * action, gpointer data)
 static void
 on_save (GtkAction * action, gpointer data)
 {
+	conversation_panel_save_current (CONVERSATION_PANEL (gui.conversation_panel));
 }
 
 static void
@@ -1559,7 +1570,7 @@ on_close (GtkAction * action, gpointer data)
 			client = gconf_client_get_default ();
 			text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
 			if (text == NULL)
-				text = g_strdup ("Ex-Chat");
+				text = g_strdup (_("Ex-Chat"));
 			s->server->p_part (s->server, s->channel, text);
 			g_object_unref (client);
 			g_free (text);
@@ -1568,18 +1579,23 @@ on_close (GtkAction * action, gpointer data)
 		}
 
 		fe_close_window (s);
-		text_gui_remove_text_buffer (s);
+		if (s->type == SESS_SERVER)
+			status_bar_remove_server (STATUS_BAR (gui.status_bar), s->server);
+		conversation_panel_remove_session (CONVERSATION_PANEL (gui.conversation_panel), s);
+		topic_label_remove_session        (TOPIC_LABEL        (gui.topic_label),        s);
+		text_entry_remove_session         (TEXT_ENTRY         (gui.text_entry),         s);
 	}
 }
 
 static void
 on_find (GtkAction * action, gpointer data)
 {
-}
+	GtkWidget *widget;
 
-static void
-on_find_again (GtkAction * action, gpointer data)
-{
+	widget = glade_xml_get_widget (gui.xml, "find hbox");
+	gtk_widget_show (widget);
+	widget = glade_xml_get_widget (gui.xml, "find entry");
+	gtk_widget_grab_focus (widget);
 }
 
 static void
@@ -1589,7 +1605,7 @@ on_clear (GtkAction * action, gpointer data)
 
 	s = navigation_tree_get_selected_session ();
 	if (s)
-		clear_buffer (s);
+		conversation_panel_clear (CONVERSATION_PANEL (gui.conversation_panel), s);
 }
 
 static void
@@ -1613,7 +1629,7 @@ on_channel_leave (GtkAction * action, gpointer data)
 			client = gconf_client_get_default ();
 			text = gconf_client_get_string (client, "/apps/xchat/irc/partmsg", NULL);
 			if (text == NULL)
-				text = g_strdup ("Ex-Chat");
+				text = g_strdup (_("Ex-Chat"));
 			s->server->p_part (s->server, s->channel, text);
 			g_object_unref (client);
 			g_free (text);
