@@ -1,7 +1,7 @@
 /*
  * palette.h - palette setting/saving functions
  *
- * Copyright (C) 2004 xchat-gnome team
+ * Copyright (C) 2004-2005-2005 xchat-gnome team
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -24,8 +24,11 @@
 extern GdkColor colors[];
 extern const GdkColor *color_schemes[];
 extern const GdkColor *palette_schemes[];
+extern GdkColor custom_colors[9];
+extern GdkColor custom_palette[32];
 
 void palette_alloc (GtkWidget *widget);
 void load_colors (int selection);
 void load_palette (int selection);
-void palette_init ();
+void palette_init (void);
+void palette_save (void);
