@@ -19,9 +19,10 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 #include <gmodule.h>
-#include <libgnome/gnome-i18n.h>
 #include <string.h>
 #include "gui.h"
 #include "../common/util.h"
@@ -36,6 +37,8 @@
 #include "xg-plugin.h"
 #include "plugins.h"
 
+xchat_gnome_plugin *new_xg_plugin (void);
+
 typedef int  (xchat_init_func)         (xchat_plugin *, char **, char **, char **, char *);
 typedef int  (xchat_deinit_func)       (xchat_plugin *);
 typedef void (xchat_plugin_get_info)   (char **, char **, char **, char **);
@@ -51,7 +54,7 @@ autoload_plugin_cb (gchar * filename, gpointer data)
 }
 
 void
-autoload_plugins ()
+autoload_plugins (void)
 {
 	/* Split plugin loading into two passes - one for plugins, one for
 	 * scripts.  This makes sure that any language binding plugins are
@@ -62,12 +65,12 @@ autoload_plugins ()
 }
 
 void
-plugins_initialize ()
+plugins_initialize (void)
 {
 	GConfClient *client;
 
 	client = gconf_client_get_default ();
-	enabled_plugins = gconf_client_get_list (client, "/apps/xchat/plugins/enabled", GCONF_VALUE_STRING, NULL);
+	enabled_plugins = gconf_client_get_list (client, "/apps/xchat/plugins/loaded", GCONF_VALUE_STRING, NULL);
 }
 
 int
@@ -80,26 +83,21 @@ unload_plugin (char *filename)
 		return plugin_kill (filename, 1);
 	} else {
 		/* Script. */
-		char *buf = (char*) malloc (len + 10);
-
-		if (strchr (filename, ' '))
-			sprintf (buf, "UNLOAD \"%s\"", filename);
-		else
-			sprintf (buf, "UNLOAD %s", filename);
-
-		handle_command (gui.current_session, buf, FALSE);
-		free (buf);
+		gchar *command = g_strdup_printf ("UNLOAD \"%s\"", filename);
+		handle_command (gui.current_session, command, FALSE);
+		g_free (command);
 	}
 
 	return 1;
 }
 
 xchat_gnome_plugin *
-new_xg_plugin ()
+new_xg_plugin (void)
 {
-	xchat_gnome_plugin *plugin = malloc (sizeof (xchat_gnome_plugin));
+	xchat_gnome_plugin *plugin = g_new0 (xchat_gnome_plugin, 1);
 	plugin->xg_get_main_window = xg_get_main_window;
-	plugin->xg_get_chan_list = xg_get_chan_list;
+	plugin->xg_get_chan_list   = xg_get_chan_list;
+	plugin->xg_get_ui_manager  = xg_get_ui_manager;
 
 	return plugin;
 }
@@ -108,7 +106,7 @@ char *
 load_plugin (session * sess, char *filename, char *arg, gboolean script, gboolean autoload)
 {
 	int len;
-	char *buf, *err;
+	char *err;
 	void *handle;
 	gpointer xg_init_func;
 	xchat_gnome_plugin *pl;
@@ -132,14 +130,9 @@ load_plugin (session * sess, char *filename, char *arg, gboolean script, gboolea
 		}
 	} else if (script == TRUE) {
 		/* Script */
-		buf = (char*) malloc (len + 9);
-		if (strchr (filename, ' '))
-			sprintf (buf, "LOAD \"%s\"", filename);
-		else
-			sprintf (buf, "LOAD %s", filename);
-
-		handle_command (gui.current_session, buf, FALSE);
-		free (buf);
+		gchar *command = g_strdup_printf ("LOAD \"%s\"", filename);
+		handle_command (gui.current_session, command, FALSE);
+		g_free (command);
 	}
 
 	return NULL;
@@ -148,13 +141,19 @@ load_plugin (session * sess, char *filename, char *arg, gboolean script, gboolea
 
 /*** Functions called by plugins ***/
 GtkWidget *
-xg_get_main_window ()
+xg_get_main_window (void)
 {
 	return (GTK_WIDGET (gui.main_window));
 }
 
 GtkTreeModel *
-xg_get_chan_list ()
+xg_get_chan_list (void)
 {
 	return gtk_tree_view_get_model (GTK_TREE_VIEW (gui.server_tree));
 }
+
+GtkUIManager *
+xg_get_ui_manager (void)
+{
+	return g_object_ref (gui.manager);
+}
