@@ -19,8 +19,9 @@
  *
  */
 
+#include <config.h>
+#include <glib/gi18n.h>
 #include "preferences-dialog.h"
-#include "preferences-keybindings-page.h"
 #include "pixmaps.h"
 
 static GtkDialogClass *parent_class;
@@ -88,8 +89,8 @@ preferences_dialog_init (PreferencesDialog *p)
 	p->toplevel = NULL;
 
 	xml = NULL;
-	if (g_file_test ("preferences-dialog.glade", G_FILE_TEST_EXISTS))
-		xml = glade_xml_new ("preferences-dialog.glade", "toplevel", NULL);
+	if (g_file_test ("../../data/preferences-dialog.glade", G_FILE_TEST_EXISTS))
+		xml = glade_xml_new ("../../data/preferences-dialog.glade", "toplevel", NULL);
 	if (!xml)
 		xml = glade_xml_new (XCHATSHAREDIR "/preferences-dialog.glade", "toplevel", NULL);
 	if (!xml)
@@ -163,7 +164,7 @@ preferences_dialog_get_type (void)
 }
 
 PreferencesDialog *
-preferences_dialog_new ()
+preferences_dialog_new (void)
 {
 	PreferencesDialog *p = g_object_new (preferences_dialog_get_type (), 0);
 	if (p->toplevel == NULL) {
