@@ -19,10 +19,13 @@
  *
  */
 
-#include "../common/xchat.h"
-#include "../common/xchatc.h"
+#include <config.h>
+#include <glib/gi18n.h>
+#include <string.h>
 #include "preferences-page-dcc.h"
 #include "preferences-dialog.h"
+#include "../common/xchat.h"
+#include "../common/xchatc.h"
 
 static void
 download_dir_changed (GtkFileChooser *file_chooser, gpointer data)
@@ -184,9 +187,12 @@ preferences_page_dcc_new (gpointer prefs_dialog, GladeXML *xml)
 	gtk_size_group_add_widget (group, page->global_receive_throttle);
 	g_object_unref (group);
 
-	page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/dcc.png", NULL);
+	if (g_file_test ("../../data/dcc.png", G_FILE_TEST_EXISTS))
+		page->icon = gdk_pixbuf_new_from_file ("../../data/dcc.png", NULL);
+	else
+		page->icon = gdk_pixbuf_new_from_file (XCHATSHAREDIR "/dcc.png", NULL);
 	gtk_list_store_append (p->page_store, &iter);
-	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, "File Transfers & DCC", 2, 3, -1);
+	gtk_list_store_set (p->page_store, &iter, 0, page->icon, 1, _("File Transfers & DCC"), 2, 3, -1);
 
 	return page;
 }
@@ -194,6 +200,6 @@ preferences_page_dcc_new (gpointer prefs_dialog, GladeXML *xml)
 void
 preferences_page_dcc_free (PreferencesDCCPage *page)
 {
-	gdk_pixbuf_unref (page->icon);
+	g_object_unref (page->icon);
 	g_free (page);
 }
